import socket
import ssl
import time

def download_page(url_id, type_id=True, split=True, method='0'):
	if method == '0':
		return download_xml(url_id, type_id, split)
	elif method == '1':
		return download_html(url_id, type_id, split)
	else:
		return None

def download_xml(url_id, type_id=True, split=True):
	"""Return a list of informations of each video with the 
	RSS youtube page"""
	nb = 0
	data = b""
	if type_id: #Channel
		url = b'/feeds/videos.xml?channel_id=' + url_id.encode()
	else: #Playlist
		url = b'/feeds/videos.xml?playlist_id=' + url_id.encode()
	data = download_http(url)
	if split:
		linfo = data.split("<entry>")
		del linfo[0]
		if linfo == []:
			return None
		return linfo
	else:
		return data

def download_html(url_id, type_id=True, split=True):
	"""Return a list of informations of each video with the
	current html page"""
	if type_id:
		url = b'/channel/' + url_id.encode() + b'/videos'
	else:
		url = b'/playlist?list=' + url_id.encode()
	data = download_https(url)
	if data == None:
		return None
	if split:
		if type_id: #channel
			linfo = data.split('<div class="yt-lockup-content">')
			if len(linfo) <= 1:
				return None
		else: #playlist
			linfo = data.split('<tr class="pl-video yt-uix-tile "')
			del linfo[0]
			if linfo == []:
				return None
		return linfo
	else:
		return data

def download_http(url):
	data = b''
	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	sock.connect(("youtube.com", 80))
	sock.send(b"GET " + url + b" HTTP/1.0\r\nHost: www.youtube.com\r\n\r\n")
	while True:
		raw_data = sock.recv(1024)
		if raw_data == b"":
			break
		else:
			data += raw_data
	sock.close()
	try:
		return data.decode('utf8')
	except:
		return None

def download_https(url):
	data = b''
	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	sock.connect(("youtube.com", 443))
	for i in range(5):
		try:
			ssock = ssl.wrap_socket(sock)
		except OSError:
			time.sleep(1)
			if i == 4:
				return None
		else:
			break
	ssock.write(b"GET " + url + b' HTTP/1.1\r\nHost: www.youtube.com\r\nAccept-Language: en\r\n\r\n')
	# Recv the HTML page :
	while True:
		try:
			raw_data = ssock.recv(1000)
		except ConnectionResetError:
			return None
		except:
			break
		data += raw_data
		if b'</html>' in data[-20:] or raw_data == b'':
			break
	ssock.close()
	try:
		return data.decode('utf8')
	except:
		return None



from .time import lcl_time
from .sock import download_xml
from threading import Thread

def old(url_data, min_tps=12):
	lcl = lcl_time(min_tps)
	threads = []
	for url in url_data:
		thr = Thread(target=thread_old, args=(url, lcl,))
		threads.append(thr)
		thr.start()
	for i in threads:
		i.join()

def thread_old(url, lcl):
	linfo = download_xml(url)
	if linfo == False:
		print('[channel dead]', url)
	elif linfo == None:
		print('[  no video  ]', url)
	else:
		tps_vd = linfo[0].split("<published>")[1].split("</published>")[0].replace('-', '').replace('+00:00', '').replace('T', '').replace(':', '')
		if lcl > int(tps_vd):
			try:
				print('[ ' + tps_vd[:4] + '/' + tps_vd[4:6] + '/' + tps_vd[6:8] + ' ] ' + linfo[0].split('<name>')[1].split('</name>')[0])
			except UnicodeEncodeError:
				print('[ ' + tps_vd[:4] + '/' + tps_vd[4:6] + '/' + tps_vd[6:8] + ' ]', linfo[0].split('<name>')[1].split('</name>')[0].encode())
			except:
				print("[   erreur   ]", url)

def dead(url_data):
	threads = []
	for url in url_data:
		thr = Thread(target=thread_dead, args=(url,))
		threads.append(thr)
		thr.start()
	for i in threads:
		i.join()

def thread_dead(url):
	linfo = download_xml(url)
	if linfo == False:
		print('[channel dead]\t', url)
	elif linfo == None:
		print('[  no video  ]\t', url)


def stats(subs):
	for sub in subs:
		threads = []
		thr = Thread(target=stat, args=(sub, subs[sub],))
		threads.append(thr)
		thr.start()
	for i in threads:
		i.join()


def stat(sub, name=''):
	"""Print the mark and the views of a channel"""
	data = download_xml(sub)
	if data == None or data == False:
		return
	marks = 0
	views = 0
	count_marks = 0
	for i in data:
		mark, view = recup_stats(i)
		if mark != None:
			marks += mark
			count_marks += 1
		views += view
	marks = marks / (count_marks / 2)
	prin = str(marks)[:4] + " for " + name + ' (' + str(views) + ' views)'
	try:
		print(prin)
	except:
		print(prin.encode())

def recup_stats(data):
	raw = data.split("<media:community>")[1].split("</media:community>")[0]
	try:
		mark = float(data.split("average=\"")[1].split("\"")[0])
	except:
		mark = None
	try:
		view = int(data.split("views=\"")[1].split("\"")[0])
	except:
		view = 0
	return mark, view



from time import localtime

def lcl_time(nb_month=1, all_time=False):
	if all_time:
		return 0
	tps = localtime()
	tm_year, tm_mon, tm_mday = tps.tm_year, tps.tm_mon, tps.tm_mday
	if tm_mon <= nb_month:
		if nb_month >= 12:
			tm_year -= int(nb_month/12)
			tm_mon -= (nb_month % 12)
		else:
			tm_mon = 12 + (tm_mon - nb_month)
			tm_year -= 1
	else:
		tm_mon -= nb_month
	return split_to_date(tm_year, tm_mon, tm_mday)

def since(day):
	tps = localtime()
	tm_year, tm_mon, tm_mday = tps.tm_year, tps.tm_mon, tps.tm_mday
	if day >= 365: #year(s)
		var = (day-(day%365))
		tm_year -= var/365
		day -= var
	if day >= 31: #month(s)
		var = (day-(day%31))
		tm_mon = tm_mon - var/31
		if tm_mon <= 0:
			tm_year -= 1
			tm_mon += 12
		day -= var
	if day >= 0: #week(s)
		tm_mday -= day
		if tm_mday <= 0:
			tm_mday += 31
			tm_mon -= 1
			if tm_mon <= 0:
				tm_year -= 1
				tm_mon += 12
		return split_to_date(tm_year, tm_mon, tm_mday, False)
	
def split_to_date(tm_year, tm_mon, tm_mday, nb=True):
	tm_mday = str(tm_mday)[:2]
	tm_mon = str(tm_mon)[:2].replace('.', '')
	tm_year = str(tm_year)[:4]
	if len(tm_mday) == 1:
		tm_mday = '0' + tm_mday
	if len(tm_mon) == 1:
		tm_mon = '0' + tm_mon
	if nb:
		return int(tm_year + tm_mon + tm_mday + '000000')
	else:
		return tm_year + tm_mon + tm_mday + '000000'


import time
import os
import sys
from .src.analyzer import (
	html_init,
	html_end,
	init,
	sort_file)
from .src.swy import (
	swy,
	generate_swy,
	add_sub)
from .src.tools import (
	del_data,
	check_id)
from .src.time import lcl_time



def init_command(path, arg):
	from shutil import rmtree
	if os.path.exists(path):
		rmtree(path)
	try:
		os.makedirs(path + 'data/')
	except:
		exit('[*] Error We Can\'t Create The Folder')
	else:
		print('[*] Data File Create')
	if len(sys.argv) != arg + 1:
		add_file = sys.argv[arg + 1]
		if not os.path.exists(add_file):
			exit('[!] File Not Found (' + add_file + ')')
		if add_file[len(add_file) - 4:] == '.swy':
			with open(add_file, 'rb') as file, open(path + 'sub.swy', 'a', encoding='utf8') as sub_file:
				nb_line = 1
				while True:
					line = file.readline().decode('utf8')
					if '\t' in line:
						sub_file.write(line)
					elif line == '':
						break
					else:
						print('[!] No tabs in line ' + str(nb_line))
		else:
			url_data = swy(add_file, path=path)
	else:
		if os.path.exists('subscription_manager'):
			url_data = swy('subscription_manager', path=path)
		else:
			exit('[!] File Not Found (subscription_manager)')
	print('[*] Subs append to sub.swy')
	exit('[*] Done')

def main():
	url_data = []
	analyze = False
	analyze_only_one = False
	loading = False
	method = '0'
	mode = 'html'
	count = 7
	all_time = False
	output = ''
	sub_file = 'subscription_manager'
	#path
	if os.name == 'nt':
		path = os.path.expanduser('~') + '/.youtube_sm/'
	else:
		if os.uname().sysname == 'Linux':
			path = os.environ['HOME'] + '/.cache/youtube_sm/'
	try:
		os.makedirs(path + 'data/')
	except:
		pass
	#commands
	del sys.argv[0]
	if sys.argv==[]:
		passe = time.time()
		url_data = swy(path=path)
		html_init(path)
		init(url_data, 'sub.html', lcl_time(), path)
		html_end(path=path)
		open(path + 'log', 'a').write(str(time.time() - passe) + '\t' + time.strftime("%H%M") + '\n')
	elif sys.argv == ['-h'] or sys.argv == ['--help']:
		print("""Usage: youtube-sm [OPTIONS]

Options:
   -h                     Print this help text and exit
   -n     [file]          Use an other xml file for your subscriptions
   -m     [mode]          Choose the type of the output (html, raw, list)
   -t     [nb of days]    Choose how far in the past do you want the program to look for videos
   -d                     Show the dead channels + those who posted no videos
   -o     [nb of months]  Show the channels who didn't post videos in [nb of months] + dead channels
   -l     [id]            Analyze only one channel or playlist
   -r                     Remove the cache
   -s     [id/all]        Outputs the stats of the selected channel(s)
   -a     [id]            Append a channel or a playlist at the end of sub.
   --init [file]          Remove all your subs and the cache and init with your subscription file.
   --af   [file]          Append a file with list of channel or a playlist in sub.swy
   --ax   [file]          Append a xml file in sub.swy
   --html                 Recover yours subs in the common page web (more videos)
   --output [file]        Choose the name of the output file
   --cat                  View your subscriptions
   --css                  Import the css files
   --loading              Prints a progress bar while running
""", end='')
	else:
		for arg in range(len(sys.argv)):
			if sys.argv[arg][0] != '-':
				continue
			if sys.argv[arg][1] != '-':
				if sys.argv[arg] == '-o':
					from .src.channel_analyzer import old
					url_data = swy(path=path)
					print('[*]Start of analysis')
					try:
						min_tps = int(sys.argv[arg + 1])
					except:
						old(url_data)
					else:
						old(url_data, min_tps)
				elif sys.argv[arg] == '-d':
					from .src.channel_analyzer import dead
					url_data = swy(path=path)
					print('[*]Start of analysis')
					dead(url_data)
				elif sys.argv[arg] == '-m':
					analyze = True
					if sys.argv[arg + 1] in ['html', 'raw', 'list']:
						mode = sys.argv[arg + 1]
					else:
						exit('[!] Mode file invalid')
				elif sys.argv[arg] == '-t':
					analyze = True
					try:
						count = int(sys.argv[arg + 1])
						if count == -1:
							all_time = True
					except:
						exit('[!] Numbers of day invalid')
				elif sys.argv[arg] == '-n':
					if os.path.exists(sys.argv[arg + 1]):
						if os.path.exists(path + 'sub.swy'):
							os.remove(path + 'sub.swy')
						generate_swy(sys.argv[arg + 1], path)
					else:
						exit('[!] File not found')
				elif sys.argv[arg] == '-a':
					if sys.argv[arg + 1][:2] in ['UC', 'PL']:
						add_sub([sys.argv[arg + 1]], path)
					else:
						print('[!] Id is not available')
				elif sys.argv[arg] == '-l':
					analyze = True
					analyze_only_one = True
					del_data(path, False)
					if arg + 1 >= len(sys.argv):
						exit('[!] You forgot an argument after -l')
					if sys.argv[arg + 1][:2] in ['UC', 'PL']:
						url_data = [sys.argv[arg + 1]]
					else:
						exit('[!] Id is not available')
				elif sys.argv[arg] == '-r':
					del_data(path, True)
				elif sys.argv[arg] == '-s':
					try:
						if check_id(sys.argv[arg+1]):
							from .src.channel_analyzer import stat
							stat(sys.argv[arg+1])
						elif sys.argv[arg+1] == 'all':
							from .src.channel_analyzer import stats
							subs = swy(path=path, liste=False)
							stats(subs)
						else:
							exit("[!] Id is not available")
					except IndexError:
						exit("[!] Missing argument after the '-s'")
				elif sys.argv[arg] == '-h':
					exit("[!] -h don't work with other options")
				elif sys.argv[arg] == '-1':
					pass
				else:
					exit("[!] No such option: {}".format(sys.argv[arg]))
			else:
				if sys.argv[arg] == '--af':
					if os.path.exists(sys.argv[arg + 1]):
						add_sub(open(sys.argv[arg + 1], 'r').read().split('\n'), path)
					else:
						exit('[!] File not found')
				elif sys.argv[arg] == '--ax':
					if os.path.exists(sys.argv[arg + 1]):
						generate_swy(sys.argv[arg + 1], path)
					else:
						exit('[!] File not found')
				elif sys.argv[arg] == '--cat':
					if os.path.exists(path + 'sub.swy'):
						with open(path + 'sub.swy', 'r') as file:
							while True:
								line = file.readline()
								if line == '':
									break
								try:
									print(line, end='')
								except:
									print(line.encode())
				elif sys.argv[arg] == '--html':
					method = '1'
					analyze = True
				elif sys.argv[arg] == '--css':
					try:
						os.mkdir('css')
					except:
						pass
					open('css/sub.css', 'a', encoding='utf8').write(".left\n{\n	float: left;\n	\n}\n.clear\n{\n	clear: both;\n}\n*\n{\n	font-family: Arial;\n}\n\ndiv\n{\n	margin: 10px 27% 10px 27%;\n}\nimg\n{\n	width: 280px;\n	height: 157px;\n	margin-right: 7px;\n}\nh4\n{\n	line-height: 18px;\n	font-size: 18px;\n	margin: 0px 0px -5px 0px;\n}\np\n{\n	color: grey;\n	line-height: 7px;\n}\na\n{\n	text-decoration: none;\n	color: black;\n}\n")
					open('css/sub_mobile.css', 'a', encoding='utf8').write(".left\n{\n	float:left;\n}\n.clear\n{\n	clear: both;\n}\n*\n{\n	font-family: Arial;\n}\ndiv\n{\n	margin-left: 0%;\n	margin-right: 0%;\n	margin: 10px 0px 10px 0px;\n}\nimg\n{\n	width: 380px;\n	height: 214px;\n}\nh4\n{\n	line-height: 30px;\n	font-size: 30px;\n	margin: 0px 0px -10px 0px;\n}\np\n{\n	color: grey;\n	line-height: 5px;\n	font-size: 1.9em;\n}\na\n{\n	text-decoration: none;\n	color: black;\n}")
				elif sys.argv[arg] == '--init':
					init_command(path, arg)
				elif sys.argv[arg] == '--loading':
					loading = True
					analyze = True
				elif sys.argv[arg] == '--output':
					if arg + 1 >= len(sys.argv):
						exit('[!] You forgot an argument after --output')
					output = sys.argv[arg+1]
					analyze = True
				elif sys.argv[arg] == '--help':
					exit("[!] -h don't work with other options")
				else:
					exit("[!] No such option: {}".format(sys.argv[arg]))
	if analyze:
		passe = time.time()
		if not analyze_only_one:
			url_data = swy(sub_file, path=path)
		if output == '':
			if mode == 'html':
				output = 'sub.html'
			elif mode == 'list':
				output = 'sub_list'
			elif mode == 'raw':
				output = 'sub_raw'
		if mode == 'html':
			html_init(path, output)
		elif mode == 'raw' or mode == 'list':
			if os.path.exists(output):
				os.remove(output)			
		nb_new = init(url_data, output, lcl_time(int(count/30+(30-count%30)/30), all_time), path, mode, loading, method)
		sort_file(count, output, mode, path, method)
		open(path + 'log', 'a').write(str(time.time() - passe) + '\t' + time.strftime("    %H%M") + '\n')



class Progress():
	"""Print a progress bar"""
	def __init__(self, xmax=0):
		self.xmin = 0
		self.xmax = xmax

	def add(self):
		self.xmin += 1
		self.progress_bar()

	def progress_bar(self):
		load = ''
		if self.xmax == 0:
			pc = 0
		else:
			pc = (self.xmin/self.xmax)
		for i in range(int(pc*40)):
			load += '█'
		for i in range(int(40 - pc*40 + (pc*40)%1)):
			load += ' '
		print('{} %|{}| {}/{} analyzed'.format(str(pc*100)[:3], load, str(self.xmin), str(self.xmax)), end='\r')
		if pc == 1:
			print()

def type_id(id):
	"""True = Channel; False = Playlist"""
	if id[:2] == 'UC':
		return True
	elif id[:2] == 'PL':
		return False
	else:
		return True

def check_id(id):
	if id[:2] == 'UC' or id[:2] == 'PL':
		return True
	else:
		return False

def del_data(path='', prin=True):
	from shutil import rmtree
	import os
	if os.path.exists(path + 'data'):
		for i in range(2):
			try:
				rmtree(path + 'data')
			except:
				pass
			else:
				continue
		try:
			os.makedirs(path + 'data/')
		except:
			pass
	else:
		if prin:
			print('[!] Data do not exist')



from os.path import exists
from .sock import download_xml
from .tools import type_id

def generate_swy(sub_file, path=''):
	"""Add sub in sub.swy"""
	if exists(sub_file):
		data = open(sub_file, 'r', encoding="utf8").read()
	else:
		exit('[!] File not found (' + sub_file + ')')
	liste = data.split('<outline')
	del liste[:2]
	for i in liste:
		channel = i.split('title="')[1].split('"')[0]
		id_chnl = i.split('xmlUrl="')[1].split('"')[0].replace('https://www.youtube.com/feeds/videos.xml?channel_id=', '')
		open(path + 'sub.swy', 'a', encoding='utf8').write('{}\t{}\n'.format(id_chnl, channel))


def add_sub(subs, path=''):
	"""Add a list of subs in sub.swy"""
	for i in subs:
		tid = type_id(i)
		data = download_xml(i, type_id=tid, split=False)
		if data == None:
			continue
		try:
			if tid:
				data = data.split('<name>')[1].split('</name>')[0]
			else:
				data = data.split('<title>')[1].split('</title>')[0]
			open(path + 'sub.swy', 'a', encoding='utf8').write(i + '\t' +  data + '\n')
		except:
			pass

def swy(sub_file='subscription_manager', path='', liste=True):
	"""Generate a list of subs which are append in sub.swy and return"""
	if not exists(path + 'sub.swy'):
		generate_swy(sub_file, path)
	data_sub = open(path + 'sub.swy', 'rb').read().decode("utf8").split('\n')
	if liste:
		url_data = []
		for i in data_sub:
			url_data.append(i.split('\t')[0])
	else:
		url_data = dict()
		for i in data_sub:
			ch = i.split('\t')
			try:
				url_data[ch[0]] = ch[1][:len(ch[1])-1]
			except:
				pass
	return url_data




from setuptools import setup, find_packages

params = dict()
params['entry_points'] = {'console_scripts': ['youtube-sm = youtube_sm.commands:main']}

setup(
	name='youtube_sm',
	version='1.1.2',
	url='https://github.com/sawyerf/Youtube_subscription_manager',
	author='Sawyerf',
	author_email='sawyer.flink@protonmail.ch',
	description='Youtube subscription manager',
	long_description=open('README.md', 'r', encoding='utf8').read().replace('\r', ''),
	long_description_content_type='text/markdown',
	keywords='youtube subscription api manager html',
	license='MLP-2.0',
	packages=find_packages(),
	classifiers=[
		'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
		'Environment :: Console',
		'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
		'Operating System :: Microsoft :: Windows',
		'Operating System :: POSIX :: Linux',
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3 :: Only'
	],
	**params
)



import os
import time
import re

from threading import Thread
from .sock import (
	download_xml,
	download_html)
from .tools import (
	Progress,
	type_id)
from .time import since


def html_init(path, output='sub.html'):
	"""To init the html file"""
	open(output, 'w').write("""<html>
	<head>
		<meta charset="utf-8" />
		<link rel="stylesheet" href="css/sub.css" />
		<link rel="stylesheet" media="screen and (max-width: 1081px)" href="css/sub_mobile.css"/>
		<title>Abonnements</title>
	</head>
	<body>
<!-- {} -->
""".format(time.ctime()))

def init(urls, output, min_date, path='', mode='html', loading=False, method='0'):
	"""Run all the analyze in a thread"""
	threads = []
	ending = 0
	nb = len(urls)
	if loading:
		prog = Progress(nb)
		prog.progress_bar()
	if method == '0':
		max_thr = 200
	elif method == '1':
		max_thr = 50
	for i in range(nb):
		if loading:
			thr = Analyzer(mode, urls[i], output, min_date, method, path, prog)
		else:
			thr = Analyzer(mode, urls[i], output, min_date, method, path)
		thr.Thread()
		threads.append(thr)
		thr.start()
		if i%max_thr == 0 or nb == i+1:
			for y in threads:
				y.join()
			threads = []
	if loading and prog.xmin != prog.xmax:
		prog.xmin = prog.xmax
		prog.progress_bar()


class Analyzer(Thread):
	"""It's a group of fonctions to recup the videos informations"""
	def __init__(self, mode='', url_id='', output='', min_date=0, method='0', path='', prog=None):
		self.mode = mode
		# self.method is the way you're going to recv the data (0 -> RSS, 1 -> https://youtube.com/channel/{id}/videos) 
		self.method = method
		self.id = url_id
		if output != '':
			self.output = output
		else:
			if mode == 'html':
				self.output = 'sub.html'
			elif mode == 'list':
				self.output = 'sub_list'
			elif mode == 'raw':
				self.output = 'sub_raw'
			else:
				self.output = 'sub'
		self.path_cache = path
		self.min_date = min_date
		self.type = self._type_id()
		#all the var user in class
		self.url = ""
		self.url_channel = ""
		self.title = ""
		self.channel = ""
		self.date = ""
		self.data_file = ""
		#Progress
		self.prog = prog

	def Thread(self):
		Thread.__init__(self)

	def run(self):
		self.analyzer_sub()
		if self.prog != None:
			self.prog.add()

	def _type_id(self):
		return type_id(self.id)

	def _download_page(self):
		if self.method == '0':
			return download_xml(self.id, self.type)
		elif self.method == '1':
			return download_html(self.id, self.type)
		else:
			return None

	def analyzer_sub(self):
		"""Recover all the videos of a channel or a playlist
		and add the informations in $HOME/.cache/youtube_sm/data/."""
		linfo = self._download_page()
		if linfo == False or linfo == None:
			return
		try:
			os.mkdir(self.path_cache + 'data/' + self.method)
		except:
			pass
		if self.method == '0':
			for i in linfo:
				date = int(i.split("<published>")[1].split("</published>")[0].replace('-', '').replace('+00:00', '').replace('T', '').replace(':', ''))
				if self.min_date <= date:
					self.info_recup_rss(i)
					self.write()
		elif self.method == '1':
			if self.type: #Channel
				try:
					self.channel = linfo[0].split('<title>')[1].split('\n')[0]
				except IndexError:
					pass
				except:
					pass
				del linfo[0]
			for i in linfo:
				try:
					self.info_recup_html(i)
				except:
					pass
				else:
					self.write()

	def _url(self, i):
		if self.method == '0':
			self.url = i.split('<yt:videoId>')[1].split('</yt:videoId>')[0]
		elif self.method == '1':
			if self.type:
				self.url = i.split('href="/watch?v=')[1].split('" rel')[0]
			else:
				self.url = i.split('data-video-id="')[1].split('"')[0]

	def info_recup_html(self, i):
		"""Recover the informations of the html page"""
		if self.type: #Channel
			self.url = re.findall(r'href="/watch\?v=(.*)" rel', i)[0]
			self.url_channel = self.id
			self.title = re.findall(r'dir="ltr" title="(.*)"  aria', i)[0]
			self.date = re.findall(r'</li><li>(.*)</li>', i)[0]
			self.data_file = [self.date_convert(), 'no_hour']
		else: #Playlist
			self.url = re.findall(r'data-video-id="(.*)"', i)[0]
			if '<a href="/user/' in i:
				self.url_channel = re.findall('<a href="/user/(.*)"', i)[0]
			elif '<a href="/channel/' in i:
				self.url_channel = re.findall('<a href="/channel/(.*)"', i)[0]
			self.title = re.findall('data-title="(.*)"', i)[0]
			self.channel = i.split('</a>')[2].split('" >')[1]
			self.data_file = ['Playlist', self.id]

	def info_recup_rss(self, i):
		"""Recover the informations of a rss page"""
		self.url = re.findall(r'<yt:videoId>(.*)</yt:videoId>', i)[0]
		self.url_channel = re.findall(r'<yt:channelId>(.*)</yt:channelId>', i)[0]
		self.title = re.findall(r'<media:title>(.*)</media:title>', i)[0]
		self.channel = re.findall(r'<name>(.*)</name>', i)[0]
		self.data_file = re.findall(r'<published>(.*)\+', i)[0].split('T')
		self.date = self.data_file[0]
		self.data_file[0] = self.data_file[0].replace('-', '')

	def date_convert(self):
		sdate = self.date.split(' ')
		if 'year' in sdate[1]:
			day = 365*int(sdate[0])
			return since(day)[:4] + '0000'
		elif 'month' in sdate[1]:
			day = 31*int(sdate[0])
			return since(day)[:6] + '00'
		elif 'week' in sdate[1]:
			day = 7*int(sdate[0])
		elif 'day' in sdate[1]:
			day = int(sdate[0])
		elif 'hour' in sdate[1]:
			day = 0
		elif 'minute' in sdate[1]:
			day = 0
		else:
			return '0'
		return since(day)[:8]

	def write(self):
		"""Write the information in a file"""
		if self.mode == 'html':
			return self.generate_data_html()
		elif self.mode == 'raw':
			return self.append_raw()
		elif self.mode == 'list':
			return self.append_list()

	def append_raw(self):
		"""Append the informations wich are been recover 
		in the file 'sub_raw'."""
		if self.method == '0':
			var = self.data_file[0] + self.data_file[1].replace(':', '') + '\t' + self.date + '\t' + self.url + '\t' + self.url_channel + '\t' + self.title + '\t' + self.channel + '\thttps://i.ytimg.com/vi/{}/mqdefault.jpg'.format(self.url) + '\n'
			if len(var) > 350:
				return False
			open(self.output, 'a', encoding='utf8').write(var)
		elif self.method == '1':
			var = self.data_file[0] + '000000' + '\t' + self.url + '\t' + self.url_channel + '\t' + self.title + '\t' + self.channel + '\thttps://i.ytimg.com/vi/{}/mqdefault.jpg'.format(self.url) + '\n'
			if len(var) > 350:
				return False
			open(self.output, 'a', encoding='utf8').write(var)
		var = ""
		return True

	def append_list(self):
		""""Append the informations wich are been recover
		in the file 'sub_raw'. The date is add to sort the
		videos, it is deleted"""
		if len(self.url) != 11:
			return False 
		if self.method == '0':
			open(self.output, 'a', encoding='utf8').write(self.data_file[0] + self.data_file[1].replace(':', '') + ' https://www.youtube.com/watch?v=' + self.url + '\n')
		elif self.method == '1':
			if self._type_id:
				open(self.output, 'a', encoding='utf8').write(self.data_file[0] + '000000' + ' https://www.youtube.com/watch?v=' + self.url + '\n')
			else:
				open(self.output, 'a', encoding='utf8').write('00000000000000' + ' https://www.youtube.com/watch?v=' + self.url + '\n')
		return True

	def generate_data_html(self):
		"""Append the informations wich are been recover
		in a file in '.../data/[date]/.' """
		try:
			data = open('{}data/{}/{}/{}'.format(self.path_cache, self.method, self.data_file[0], self.data_file[1].replace(':', '')), 'rb+').read().decode("utf8")
			if self.url in data:
				return False
		except:
			try:
				os.mkdir(self.path_cache + 'data/' + self.method + '/' + self.data_file[0])
			except:
				pass
		if self.url_channel[:2] == 'UC':
			self.url_channel = 'channel/' + self.url_channel
		else:
			self.url_channel = 'user/' + self.url_channel
		open('{}data/{}/{}/{}'.format(self.path_cache, self.method, self.data_file[0], self.data_file[1].replace(':', '')), 'a', encoding='utf-8').write("""<!--NEXT -->
<div class="video">
	<a class="left" href="https://www.youtube.com/watch?v={}"> <img src="https://i.ytimg.com/vi/{}/mqdefault.jpg" ></a>
	<a href="https://www.youtube.com/watch?v={}"><h4>{}</h4> </a>
	<a href="https://www.youtube.com/{}"> <p>{}</p> </a>
	<p>{}</p>
	<p class="clear"></p>
</div>
""".format(self.url, self.url, self.url, self.title, self.url_channel, self.channel, self.date))
		return True

def sort_file(count=7, output='sub.html', mode='html', path='', method='0'):
	if mode == 'html':
		html_end(count, path, output, method)
	elif mode == 'list' or mode == 'raw':
		raw_list_end(count, output)

def html_end(count=7, path='', output='sub.html', method='0'):
	"""Recover the file in '.../data/.' with all the
	informations, sort by date and add the informations
	in './sub.html'. """
	fch = sorted(os.listdir(path + 'data/' + method + '/'))
	if len(fch) < count:
		date = 0
	elif count == -1:
		date = 0
	else:
		date = int(since(count)[:8])
	sub_file = open(output, 'a', encoding='utf-8')
	i = -1
	while True:
		i += 1
		try:
			folder_date = int(fch[-1-i])
		except IndexError:
			break
		except:
			continue
		if folder_date > date:
			pass
		else:
			break
		fch_in = sorted(os.listdir(path + 'data/' + method + '/' + fch[-1-i]))
		for a in range(len(fch_in)):
			data = open(path + 'data/' + method + '/' + fch[-1-i] + '/' + fch_in[-1-a], 'r', encoding='utf-8').read()
			sub_file.write(data)
	sub_file.close()
	open(output, 'a').write('</body></html>')

def raw_list_end(count=7, output='sub'):
	"""Sorted the videos by date"""
	linfo = sorted(open(output, 'rb').read().decode('utf8').replace('\r', '').split('\n'))
	fichier = open(output, 'w', encoding='utf8')
	if count == -1:
		date = 0
	else:
		date = int(since(count)[:8])
	i = -1
	while True:
		i += 1
		try:
			folder_date = int(linfo[-1-i][:8])
		except IndexError:
			break
		except:
			continue
		if folder_date > date:
			fichier.write(linfo[-1-i][15:] + '\n')
		else:
			break
